#include <cstdlib>          // abs(int)
#include <iostream>
#include <string>
#include <vector>

// Die folgende Funktion würde auch mit einer Parameterliste (T a, T b)
// arbeiten, d.h. einer  Kopie per Wert. Da T aber für einen beliebigen
// Datentyp steht, wird eine Referenz bevorzugt,
// um Kopien von möglicherweise sehr großen Objekten zu vermeiden.
template <typename T> bool kleiner(const T &a, const T &b)          // Vergleich
{ 
  return a < b;          // zu < siehe auch Text am  Abschnittsende
}

template <> bool kleiner<int>(const int &a, const int &b)
{
  //  Das <int> in kleiner<int> darf weggelassen werden (Typdeduktion).
  return abs(a) < abs(b);          // Vergleich nach dem Absolutbetrag!
}

template <typename T> void drucke(const std::vector<T> &V)
{
  for (const T &wert : V) {
    std::cout << wert << " ";
  }
  std::cout << '\n';
}

template <typename T> T groessterWert(const std::vector<T> &vek)
{
  T max{vek[0]};          // vek darf nicht leer sein
  for (const auto &element : vek) {
    if (kleiner(max, element)) {
      max = element;
    }
  }
  return max;
}

int main()
{
  std::vector<int> iV{-100, 22, -3, 44, 6, -9, -2, 1, 8, 9};
  // In den folgenden beiden Anweisungen werden vom Compiler, gesteuert durch
  // den
  //  Datentyp vector<int> des Parameters iV, aus den obigen
  //  Templates die Funktionen groessterWert(const vector<int>&) und
  // drucke(const vector<int>&) erzeugt, ebenso wie die implizit
  // aufgerufenen Funktion kleiner(const int&, const int&).
  std::cout << "Der größte Absolutbetrag von ";
  drucke(iV);
  std::cout << "ist " << abs(groessterWert(iV)) << '\n';
  std::vector<double> dV{1.09, 2.2, 79.6, -1.9, 2.7, 100.9, 18.8, 99.9};
  // Generierung der überladenen Funktionen groessterWert(const vector<double>&)
  //  und drucke(const vector<double>&) (und der
  // aufgerufenen Funktion kleiner(const double&, const double&)):
  std::cout << "Der größte Wert von ";
  drucke(dV);
  std::cout << "ist " << groessterWert(dV) << '\n';
  // Das funktioniert auch für Strings:
  std::vector<std::string> sV{"alpha", "beta", "zebra", "eins", "elf"};
  std::cout << "Der größte Wert von ";
  drucke(sV);
  std::cout << "ist " << groessterWert(sV) << '\n';
}          // Ende von main()

