#include <algorithm>
#include <cassert>
#include <cmath>
#include <compare>
#include <iostream>
#include <vector>

class Punkt {
public:
  constexpr Punkt(double x, double y) 
  : x_(x), y_(y) 
  {}

  constexpr double x() const { return x_;}
  constexpr double y() const { return y_;}

  constexpr auto operator<=>(const Punkt& rhs) const 
  { 
    // sqrt() kann entfallen, weil die Größenrelation dadurch nicht geändert wird.
    auto abstand2_this { x_*x_ + y_*y_};     // Abstandsquadrate
    auto abstand2_rhs { rhs.x_*rhs.x_ + rhs.y_*rhs.y_};
    if (abstand2_this < abstand2_rhs) {
      return std::weak_ordering::less;
    } else if (abstand2_this > abstand2_rhs) {
      return std::weak_ordering::greater;
    }
    return std::weak_ordering::equivalent;
  }

  constexpr bool operator==(const Punkt& rhs) const 
  {
    return operator<=>(rhs) == 0;
  }
private:
  double x_;
  double y_;
};

int main()
{
  std::vector<Punkt> punkte { {1.2, 2.1}, {2.1, 1.2}, {1.0, 1.0}, 
                              {2.2, 2.2}, {2.0, 0.2}, {2.1, 1.2},
                              {0.2, 2.0}, {0.0, 0.0}, {0.1, 1.0}};
  // Test für zwei *verschiedene* Punkte mit *demselben Abstand* zum Nullpunkt:
  assert(punkte[0] == punkte[1]);  
  assert(!(punkte[0] != punkte[1]));  

  std::cout.setf(std::ios::fixed);
  std::cout.precision(1);
  std::cout << "unsortiert:\n";
  for(auto p : punkte) {
    std::cout << '(' << p.x() << ", " << p.y() <<") "; 
  }
  std::sort(punkte.begin(), punkte.end());
  std::cout << "\nnach Abstand sortiert:\n";
  for(auto p : punkte) {
    std::cout << '(' << p.x() << ", " << p.y() <<") "; 
  }
  std::cout << "\nDie Abstände nach dem Sortieren:\n";
  for(auto p : punkte) {
    std::cout << std::sqrt(p.x()*p.x() +  p.y()*p.y()) << ' '; 
  }
  std::cout << '\n';
}

