#include <iostream>
#include <cassert>
#include <fstream>
#include <source_location>
#include <string>
#include <system_error>
#include <vector>
using namespace std;

template <typename T> 
void writeVector(const std::string& filename, const std::vector<T>& v)
{
  std::ofstream dest(filename, std::ios::binary);
  if (!dest) {
    const auto& wo {source_location::current()};
    throw  std::ios::failure(std::string(wo.file_name()) + ", Zeile " 
                             + std::to_string(wo.line()));
  }
  dest.write(reinterpret_cast<const char*>(v.data()), v.size() * sizeof(T));
}

template <typename T>
void readVector(const std::string& filename, std::vector<T>& v)
{
  std::ifstream source(filename, std::ios::binary);
  if (!source) {
    const auto& wo {source_location::current()};
    throw  std::ios::failure(std::string(wo.file_name()) + ", Zeile " 
                             + std::to_string(wo.line()));
  }
  source.read(reinterpret_cast<char*>(v.data()), v.size() * sizeof(T));
}

int main()
{
  try {
    std::string dateiname("binaerdaten.bin");
    std::vector<double> v1{1.27, 3.4, 5.678, 9.01234, 100.836};
    writeVector(dateiname, v1);               // v1 schreiben
    decltype(v1) v2(v1.size(), 0.0);          // v2 mit 0 initialisieren
    readVector(dateiname, v2);                // v2 lesen
    assert(v1 == v2);                         // auf Gleichheit prüfen
  }
  catch(const std::ios::failure& exc) {
    cerr << exc.what() << '\n';
  }
}
